import { existsSync } from 'fs';
import { join, relative } from 'pathe';
import defu, { defuArrayFn } from 'defu';
import chalk from 'chalk';
import consola from 'consola';
import { defineNuxtModule, requireModule, addTemplate, resolvePath, createResolver, isNuxt2, installModule, addVitePlugin, addDevServerHandler } from '@nuxt/kit';
import { isAbsolute, resolve } from 'path';
import minimatch from 'minimatch';

const name = "@nuxtjs/tailwindcss-edge";
const version = "5.2.0-4af18c5";

function vitePlugin(tailwindConfig = {}, rootDir, cssPath) {
  const resolvedContent = (tailwindConfig.content || []).map((f) => !isAbsolute(f) ? resolve(rootDir, f) : f);
  return {
    name: "nuxt:tailwindcss",
    handleHotUpdate(ctx) {
      if (resolvedContent.findIndex((c) => minimatch(ctx.file, c)) === -1) {
        return;
      }
      const extraModules = ctx.server.moduleGraph.getModulesByFile(cssPath);
      const timestamp = +Date.now();
      for (const mod of extraModules) {
        ctx.server.moduleGraph.invalidateModule(mod, void 0, timestamp);
      }
      ctx.server.ws.send({
        type: "update",
        updates: Array.from(extraModules).map((mod) => {
          return {
            type: mod.type === "js" ? "js-update" : "css-update",
            path: mod.url,
            acceptedPath: mod.url,
            timestamp
          };
        })
      });
    }
  };
}

const defaultTailwindConfig = () => ({
  theme: {
    extend: {}
  },
  plugins: [],
  content: []
});

const logger = consola.withScope("nuxt:tailwindcss");
const layerPaths = (srcDir) => [
  `${srcDir}/components/**/*.{vue,js,ts}`,
  `${srcDir}/layouts/**/*.vue`,
  `${srcDir}/pages/**/*.vue`,
  `${srcDir}/composables/**/*.{js,ts}`,
  `${srcDir}/plugins/**/*.{js,ts}`,
  `${srcDir}/App.{js,ts,vue}`,
  `${srcDir}/app.{js,ts,vue}`,
  `${srcDir}/Error.{js,ts,vue}`,
  `${srcDir}/error.{js,ts,vue}`
];
const module = defineNuxtModule({
  meta: {
    name,
    version,
    configKey: "tailwindcss"
  },
  defaults: (nuxt) => ({
    configPath: "tailwind.config",
    cssPath: join(nuxt.options.dir.assets, "css/tailwind.css"),
    config: defaultTailwindConfig(),
    viewer: true,
    exposeConfig: false,
    injectPosition: 0,
    disableHmrHotfix: false
  }),
  async setup(moduleOptions, nuxt) {
    const configPaths = [];
    const contentPaths = [];
    const addConfigPath = async (path) => {
      if (typeof path === "string") {
        configPaths.push(await resolvePath(path));
      }
      if (Array.isArray(path)) {
        for (const _path of path) {
          configPaths.push(await resolvePath(_path));
        }
      }
    };
    if (nuxt.options._layers && nuxt.options._layers.length > 1) {
      for (const layer of nuxt.options._layers) {
        await addConfigPath(layer?.config?.tailwindcss?.configPath || join(layer.cwd + "tailwind.config"));
        contentPaths.push(...layerPaths(layer.cwd));
      }
    } else {
      await addConfigPath(moduleOptions.configPath);
      contentPaths.push(...layerPaths(nuxt.options.srcDir));
    }
    if (nuxt.options.dev) {
      configPaths.forEach((path) => nuxt.options.watch.push(path));
    }
    let tailwindConfig = {};
    for (const configPath of configPaths) {
      if (existsSync(configPath)) {
        const _tailwindConfig = requireModule(configPath, { clearCache: true });
        tailwindConfig = defu(_tailwindConfig, tailwindConfig);
        if (Array.isArray(tailwindConfig.purge)) {
          tailwindConfig.content = tailwindConfig.purge;
        }
      }
    }
    tailwindConfig.content = [...tailwindConfig.content || [], ...contentPaths];
    tailwindConfig = defuArrayFn(tailwindConfig, moduleOptions.config);
    if (moduleOptions.exposeConfig) {
      const resolveConfig = await import('tailwindcss/resolveConfig.js').then((r) => r.default || r);
      const resolvedConfig = resolveConfig(tailwindConfig);
      const template = addTemplate({
        filename: "tailwind.config.mjs",
        getContents: () => `export default ${JSON.stringify(resolvedConfig, null, 2)}`
      });
      addTemplate({
        filename: "tailwind.config.d.ts",
        getContents: () => 'declare const config: import("tailwindcss/tailwind-config").TailwindConfig\nexport { config as default }',
        write: true
      });
      nuxt.options.alias["#tailwind-config"] = template.dst;
    }
    await nuxt.callHook("tailwindcss:config", tailwindConfig);
    tailwindConfig._hash = String(Date.now());
    const cssPath = await resolvePath(moduleOptions.cssPath, { extensions: [".css", ".sass", ".scss", ".less", ".styl"] });
    const injectPosition = ~~Math.min(moduleOptions.injectPosition, (nuxt.options.css || []).length + 1);
    let resolvedCss;
    if (typeof cssPath === "string") {
      if (existsSync(cssPath)) {
        logger.info(`Using Tailwind CSS from ~/${relative(nuxt.options.srcDir, cssPath)}`);
        resolvedCss = cssPath;
      } else {
        logger.info("Using default Tailwind CSS file from runtime/tailwind.css");
        resolvedCss = createResolver(import.meta.url).resolve("runtime/tailwind.css");
      }
    }
    const resolvedNuxtCss = await Promise.all(nuxt.options.css.map((p) => resolvePath(p)));
    if (!resolvedNuxtCss.includes(resolvedCss)) {
      nuxt.options.css.splice(injectPosition, 0, resolvedCss);
    }
    const postcssOptions = nuxt.options.postcss || nuxt.options.build.postcss.postcssOptions || nuxt.options.build.postcss;
    postcssOptions.plugins = postcssOptions.plugins || {};
    postcssOptions.plugins["tailwindcss/nesting"] = postcssOptions.plugins["tailwindcss/nesting"] ?? {};
    postcssOptions.plugins["postcss-custom-properties"] = postcssOptions.plugins["postcss-custom-properties"] ?? {};
    postcssOptions.plugins.tailwindcss = tailwindConfig;
    if (isNuxt2()) {
      await installModule("@nuxt/postcss8");
    }
    if (nuxt.options.dev && !moduleOptions.disableHmrHotfix) {
      addVitePlugin(vitePlugin(tailwindConfig, nuxt.options.rootDir, resolvedCss));
    }
    if (nuxt.options.dev && moduleOptions.viewer) {
      const route = "/_tailwind";
      const createServer = await import('tailwind-config-viewer/server/index.js').then((r) => r.default || r);
      const { withTrailingSlash, withoutTrailingSlash } = await import('ufo');
      const _viewerDevMiddleware = createServer({ tailwindConfigProvider: () => tailwindConfig, routerPrefix: route }).asMiddleware();
      const viewerDevMiddleware = (req, res) => {
        if (req.originalUrl === withoutTrailingSlash(route)) {
          res.writeHead(301, { Location: withTrailingSlash(req.originalUrl) });
          return res.end();
        }
        _viewerDevMiddleware(req, res);
      };
      addDevServerHandler({ route, handler: viewerDevMiddleware });
      nuxt.hook("listen", (_, listener) => {
        const viewerUrl = `${withoutTrailingSlash(listener.url)}${route}`;
        logger.info(`Tailwind Viewer: ${chalk.underline.yellow(withTrailingSlash(viewerUrl))}`);
      });
    }
  }
});

export { module as default };
